@@ -1,7 +1,7 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2019, Magnus Edenhill
+ * Copyright (c) 2019-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -211,9 +211,10 @@ class PrintingSSLVerifyCb : public RdKafka::SslCertificateVerifyCb {
     CertGetNameStringA(ctx, CERT_NAME_FRIENDLY_DISPLAY_TYPE,
                        CERT_NAME_ISSUER_FLAG, NULL, issuer, sizeof(issuer));
 
-    std::cerr << "Broker " << broker_name << " (" << broker_id << "): "
-              << "certificate depth " << depth << ", X509 error " << *x509_error
-              << ", subject " << subject << ", issuer " << issuer << std::endl;
+    std::cerr << "Broker " << broker_name << " (" << broker_id
+              << "): " << "certificate depth " << depth << ", X509 error "
+              << *x509_error << ", subject " << subject << ", issuer " << issuer
+              << std::endl;
 
     if (ctx)
       CertFreeCertificateContext(ctx);
