@@ -283,6 +283,38 @@ struct rd_kafka_ConfigResource_result_s {
                               *   but with response error values. */
 };
 
+/**
+ * @brief Resource type specific to config apis.
+ */
+typedef enum rd_kafka_ConfigResourceType_t {
+        RD_KAFKA_CONFIG_RESOURCE_UNKNOWN = 0,
+        RD_KAFKA_CONFIG_RESOURCE_TOPIC   = 2,
+        RD_KAFKA_CONFIG_RESOURCE_BROKER  = 4,
+        RD_KAFKA_CONFIG_RESOURCE_GROUP   = 32,
+} rd_kafka_ConfigResourceType_t;
+
+/**
+ * @brief Maps `rd_kafka_ResourceType_t` to `rd_kafka_ConfigResourceType_t`
+ *        for Config Apis. We are incorrectly using `rd_kafka_ResourceType_t` in
+ *        both Config Apis and ACL Apis. So, we need this function to map the
+ *        resource type internally to `rd_kafka_ConfigResourceType_t`. Like the
+ *        enum value for `GROUP` is 32 in Config Apis, but it is 3 for ACL Apis.
+ */
+rd_kafka_ConfigResourceType_t
+rd_kafka_ResourceType_to_ConfigResourceType(rd_kafka_ResourceType_t restype);
+
+/**
+ * @brief Maps `rd_kafka_ConfigResourceType_t` to `rd_kafka_ResourceType_t`
+ *        for Config Apis. We are incorrectly using `rd_kafka_ResourceType_t` in
+ *        both Config Apis and ACL Apis. So, we need this function to map the
+ *        `rd_kafka_ConfigResourceType_t` internally to
+ *        `rd_kafka_ResourceType_t`. Like the enum value for `GROUP` is 32 in
+ *        Config Apis, but it is 3 for ACL Apis.
+ */
+rd_kafka_ResourceType_t rd_kafka_ConfigResourceType_to_ResourceType(
+    rd_kafka_ConfigResourceType_t config_resource_type);
+
+
 /**@}*/
 
 
@@ -495,6 +527,9 @@ struct rd_kafka_MemberDescription_s {
         char *group_instance_id;                /**< Group instance id */
         char *host;                             /**< Group member host */
         rd_kafka_MemberAssignment_t assignment; /**< Member assignment */
+        rd_kafka_MemberAssignment_t
+            *target_assignment; /**< Target assignment. `NULL` for `classic`
+                                   protocol */
 };
 
 /**
@@ -514,6 +549,8 @@ struct rd_kafka_ConsumerGroupDescription_s {
         char *partition_assignor;
         /** Consumer group state. */
         rd_kafka_consumer_group_state_t state;
+        /** Consumer group type. */
+        rd_kafka_consumer_group_type_t type;
         /** Consumer group coordinator. */
         rd_kafka_Node_t *coordinator;
         /** Count of operations allowed for topic. -1 indicates operations not
