@@ -623,7 +623,7 @@ static RD_INLINE RD_UNUSED int rd_kafka_Uuid_ptr_cmp(void *a, void *b) {
 
 rd_kafka_Uuid_t rd_kafka_Uuid_random();
 
-const char *rd_kafka_Uuid_str(const rd_kafka_Uuid_t *uuid);
+char *rd_kafka_Uuid_str(const rd_kafka_Uuid_t *uuid);
 
 unsigned int rd_kafka_Uuid_hash(const rd_kafka_Uuid_t *uuid);
 
