@@ -281,16 +281,14 @@ static ssize_t rd_kafka_transport_socket_recvmsg(rd_kafka_transport_t *rktrans,
                 else if (r == 0) {
                         /* Receive 0 after POLLIN event means
                          * connection closed. */
-                        rd_rkb_dbg(rktrans->rktrans_rkb, BROKER, "SOCKET",
-                                   "Disconnected: connection closed by "
-                                   "peer");
-                        rd_snprintf(errstr, errstr_size, "Disconnected");
+                        rd_snprintf(errstr, errstr_size,
+                                    "Disconnected: connection closed by "
+                                    "peer: receive 0 after POLLIN");
                         return -1;
                 } else if (r == -1 && rd_socket_errno == ECONNRESET) {
-                        rd_rkb_dbg(rktrans->rktrans_rkb, BROKER, "SOCKET",
-                                   "Disconnected: connection "
-                                   "reset by peer");
-                        rd_snprintf(errstr, errstr_size, "Disconnected");
+                        rd_snprintf(errstr, errstr_size,
+                                    "Disconnected: connection "
+                                    "reset by peer");
                         return -1;
                 } else if (r == -1) {
                         rd_snprintf(errstr, errstr_size, "%s",
@@ -344,10 +342,9 @@ static ssize_t rd_kafka_transport_socket_recv0(rd_kafka_transport_t *rktrans,
                 } else if (unlikely(r == 0)) {
                         /* Receive 0 after POLLIN event means
                          * connection closed. */
-                        rd_rkb_dbg(rktrans->rktrans_rkb, BROKER, "SOCKET",
-                                   "Disconnected: connection closed by "
-                                   "peer");
-                        rd_snprintf(errstr, errstr_size, "Disconnected");
+                        rd_snprintf(errstr, errstr_size,
+                                    "Disconnected: connection closed by "
+                                    "peer: receive 0 after POLLIN");
                         return -1;
                 }
 
@@ -717,11 +714,10 @@ static void rd_kafka_transport_io_event(rd_kafka_transport_t *rktrans,
 
                 if (r == 0 /* handshake still in progress */ &&
                     (events & POLLHUP)) {
-                        rd_rkb_dbg(rktrans->rktrans_rkb, BROKER, "SOCKET",
-                                   "Disconnected: during "
-                                   "SSL connection handshake");
                         rd_kafka_broker_conn_closed(
-                            rkb, RD_KAFKA_RESP_ERR__TRANSPORT, "Disconnected");
+                            rkb, RD_KAFKA_RESP_ERR__TRANSPORT,
+                            "Disconnected: during "
+                            "SSL connection handshake");
                         return;
                 }
 
@@ -743,12 +739,10 @@ static void rd_kafka_transport_io_event(rd_kafka_transport_t *rktrans,
                 }
 
                 if (events & POLLHUP) {
-                        rd_rkb_dbg(rktrans->rktrans_rkb, BROKER, "SOCKET",
-                                   "Disconnected: hung up from peer in "
-                                   "state AUTH_LEGACY");
                         rd_kafka_broker_fail(rkb, LOG_ERR,
                                              RD_KAFKA_RESP_ERR__AUTHENTICATION,
-                                             "Disconnected");
+                                             "Disconnected: hung up from "
+                                             "peer in state AUTH_LEGACY");
 
                         return;
                 }
@@ -771,11 +765,10 @@ static void rd_kafka_transport_io_event(rd_kafka_transport_t *rktrans,
                 }
 
                 if (events & POLLHUP) {
-                        rd_rkb_dbg(rktrans->rktrans_rkb, BROKER, "SOCKET",
-                                   "Disconnected: connection closed by "
-                                   "peer");
                         rd_kafka_broker_conn_closed(
-                            rkb, RD_KAFKA_RESP_ERR__TRANSPORT, "Disconnected");
+                            rkb, RD_KAFKA_RESP_ERR__TRANSPORT,
+                            "Disconnected: connection closed by "
+                            "peer: POLLHUP");
                         return;
                 }
 
