@@ -128,7 +128,7 @@ int mtx_lock(mtx_t *mtx)
 
   if (!mtx->mRecursive)
   {
-    while(mtx->mAlreadyLocked) Sleep(1); /* Simulate deadlock... */
+    rd_assert(!mtx->mAlreadyLocked); /* Would deadlock */
     mtx->mAlreadyLocked = TRUE;
   }
   return thrd_success;
@@ -176,7 +176,7 @@ int mtx_timedlock(mtx_t *mtx, const struct timespec *ts)
 
   if (!mtx->mRecursive)
   {
-    while(mtx->mAlreadyLocked) Sleep(1); /* Simulate deadlock... */
+    rd_assert(!mtx->mAlreadyLocked); /* Would deadlock */
     mtx->mAlreadyLocked = TRUE;
   }
 
@@ -403,18 +403,6 @@ int _cnd_timedwait_win32(cnd_t *cond, mtx_t *mtx, DWORD timeout)
   /* Wait for either event to become signaled due to cnd_signal() or
      cnd_broadcast() being called */
   result = WaitForMultipleObjects(2, cond->mEvents, FALSE, timeout);
-  if (result == WAIT_TIMEOUT)
-  {
-    /* The mutex is locked again before the function returns, even if an error occurred */
-    mtx_lock(mtx);
-    return thrd_timedout;
-  }
-  else if (result == (int)WAIT_FAILED)
-  {
-    /* The mutex is locked again before the function returns, even if an error occurred */
-    mtx_lock(mtx);
-    return thrd_error;
-  }
 
   /* Check if we are the last waiter */
   EnterCriticalSection(&cond->mWaitersCountLock);
@@ -434,9 +422,14 @@ int _cnd_timedwait_win32(cnd_t *cond, mtx_t *mtx, DWORD timeout)
     }
   }
 
-  /* Re-acquire the mutex */
+  /* The mutex is locked again before the function returns, even if an error occurred */
   mtx_lock(mtx);
 
+  if (result == WAIT_TIMEOUT)
+            return thrd_timedout;
+  else if (result == (int)WAIT_FAILED)
+            return thrd_error;
+
   return thrd_success;
 }
 #endif
@@ -518,7 +511,7 @@ static void _tinycthread_tss_cleanup (void) {
 
   while (_tinycthread_tss_head != NULL) {
     data = _tinycthread_tss_head->next;
-    free (_tinycthread_tss_head);
+    rd_free (_tinycthread_tss_head);
     _tinycthread_tss_head = data;
   }
   _tinycthread_tss_head = NULL;
@@ -536,7 +529,7 @@ static void NTAPI _tinycthread_tss_callback(PVOID h, DWORD dwReason, PVOID pv)
   }
 }
 
-#if defined(_MSC_VER)
+#ifdef _WIN32
   #ifdef _M_X64
     #pragma const_seg(".CRT$XLB")
   #else
@@ -544,9 +537,9 @@ static void NTAPI _tinycthread_tss_callback(PVOID h, DWORD dwReason, PVOID pv)
   #endif
   PIMAGE_TLS_CALLBACK p_thread_callback = _tinycthread_tss_callback;
   #ifdef _M_X64
-    #pragma data_seg()
-  #else
     #pragma const_seg()
+  #else
+    #pragma data_seg()
   #endif
 #else
   PIMAGE_TLS_CALLBACK p_thread_callback __attribute__((section(".CRT$XLB"))) = _tinycthread_tss_callback;
@@ -577,7 +570,7 @@ static void * _thrd_wrapper_function(void * aArg)
   arg = ti->mArg;
 
   /* The thread is responsible for freeing the startup information */
-  free((void *)ti);
+  rd_free((void *)ti);
 
   /* Call the actual client thread function */
   res = fun(arg);
@@ -598,7 +591,7 @@ int thrd_create(thrd_t *thr, thrd_start_t func, void *arg)
 {
   /* Fill out the thread startup information (passed to the thread wrapper,
      which will eventually free it) */
-  _thread_start_info* ti = (_thread_start_info*)malloc(sizeof(_thread_start_info));
+  _thread_start_info* ti = (_thread_start_info*)rd_malloc(sizeof(_thread_start_info));
   if (ti == NULL)
   {
     return thrd_nomem;
@@ -623,7 +616,7 @@ int thrd_create(thrd_t *thr, thrd_start_t func, void *arg)
   /* Did we fail to create the thread? */
   if(!*thr)
   {
-    free(ti);
+    rd_free(ti);
     return thrd_error;
   }
 
@@ -797,7 +790,7 @@ void tss_delete(tss_t key)
       _tinycthread_tss_tail = prev;
     }
 
-    free (data);
+    rd_free (data);
   }
   _tinycthread_tss_dtors[key] = NULL;
   TlsFree(key);
@@ -826,7 +819,7 @@ int tss_set(tss_t key, void *val)
   struct TinyCThreadTSSData* data = (struct TinyCThreadTSSData*)TlsGetValue(key);
   if (data == NULL)
   {
-    data = (struct TinyCThreadTSSData*)malloc(sizeof(struct TinyCThreadTSSData));
+    data = (struct TinyCThreadTSSData*)rd_malloc(sizeof(struct TinyCThreadTSSData));
     if (data == NULL)
     {
       return thrd_error;
@@ -852,7 +845,7 @@ int tss_set(tss_t key, void *val)
 
     if (!TlsSetValue(key, data))
     {
-      free (data);
+      rd_free (data);
 	  return thrd_error;
     }
   }
