@@ -2,6 +2,7 @@
  * librdkafka - Apache Kafka C library
  *
  * Copyright (c) 2012-2022, Magnus Edenhill
+ *               2025, Confluent Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -152,7 +153,8 @@ static void rebalance_cb(rd_kafka_t *rk,
         case RD_KAFKA_RESP_ERR__ASSIGN_PARTITIONS:
                 assign_cnt++;
 
-                rd_kafka_assign(rk, partitions);
+                test_consumer_assign_by_rebalance_protocol("rebalance", rk,
+                                                           partitions);
                 mtx_lock(&lock);
                 consumers_running = 1;
                 mtx_unlock(&lock);
@@ -177,7 +179,8 @@ static void rebalance_cb(rd_kafka_t *rk,
                 if (assign_cnt == 0)
                         TEST_FAIL("asymetric rebalance_cb");
                 assign_cnt--;
-                rd_kafka_assign(rk, NULL);
+                test_consumer_unassign_by_rebalance_protocol("rebalance", rk,
+                                                             partitions);
                 mtx_lock(&lock);
                 consumers_running = 0;
                 mtx_unlock(&lock);
