@@ -1,7 +1,7 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2018, Magnus Edenhill
+ * Copyright (c) 2018-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -130,11 +130,10 @@ static void do_test_implicit_ack(const char *what,
 
         rk = test_create_handle(RD_KAFKA_PRODUCER, conf);
 
-        test_create_topic(rk, topic, 1, 1);
+        test_create_topic_wait_exists(rk, topic, 1, 1, 5000);
 
         rkt = test_create_producer_topic(rk, topic, NULL);
 
-
         TEST_SAY("Producing %d messages\n", msgcnt);
         test_produce_msgs(rk, rkt, testid, -1, 0, msgcnt, NULL, 0);
 
