@@ -65,10 +65,7 @@ int main_0121_clusterid(int argc, char **argv) {
         rd_atomic32_t log_cnt;
         int cnt = 0;
 
-        if (test_needs_auth()) {
-                TEST_SKIP("Mock cluster does not support SSL/SASL\n");
-                return 0;
-        }
+        TEST_SKIP_MOCK_CLUSTER(0);
 
         /* Create two clusters */
         cluster_a = test_mock_cluster_new(1, &bootstraps_a);
